import java.io.*;
import java.util.*;
import org.apache.poi.ss.usermodel.*;

public class POICharsetEncodingOfCommentSetAuthorTest
{
	public static void main (String[] args) throws Exception
	{
		new POICharsetEncodingOfCommentSetAuthorTest ();
	}
	
	public POICharsetEncodingOfCommentSetAuthorTest () throws Exception
	{
		// create workbook
		Workbook workbook = new org.apache.poi.hssf.usermodel.HSSFWorkbook ();

		// create styles
		org.apache.poi.ss.usermodel.Font normalFont = workbook.createFont ();
			normalFont.setFontName ("宋体");
			normalFont.setFontHeightInPoints ((short)9);

		org.apache.poi.ss.usermodel.CellStyle normalStyle = workbook.createCellStyle ();
			normalStyle.setFont (normalFont);
			normalStyle.setDataFormat ((short)org.apache.poi.ss.usermodel.BuiltinFormats.getBuiltinFormat ("0.00"));
			
		// create sheet
		org.apache.poi.ss.usermodel.Sheet sheet = workbook.createSheet ("Charset Encoding Issue of Comment.SetAuthor");
		
		// add cells
		int iRow = 0, iCol = 0;
		POIAddStringCell (sheet, iRow, iCol, "text cell", normalStyle, "This is a text cell 文本单元格", null);	iCol++;
		POIAddNumberCell (sheet, iRow, iCol, 123.45, normalStyle, "This is a number cell 数值单元格", "ISO-8859-1");	iCol++;
		POIAddFormulaCell (sheet, iRow, iCol, "B1+100", normalStyle, "This is a formula cell 公式单元格", "GBK");	iCol++;
		
		iRow ++; iCol = 0;
		POIAddStringCell (sheet, iRow, iCol, "text cell", normalStyle, "This is a text cell 文本单元格", "UTF-8");	iCol++;
		POIAddNumberCell (sheet, iRow, iCol, 999.99, normalStyle, "This is a number cell 数值单元格", "UTF-16LE");	iCol++;
		POIAddFormulaCell (sheet, iRow, iCol, "B2+100", normalStyle, "This is a formula cell 公式单元格", "UTF-16");	iCol++;
		/*
		*/
		
		// write file
		FileOutputStream os = new FileOutputStream ("poi-comment-setAuthor-charset-encoding.xls");
		workbook.write (os);
		os.close ();
	}
	
	final Map<org.apache.poi.ss.usermodel.Sheet, Drawing> drawingPatriarches = new HashMap<org.apache.poi.ss.usermodel.Sheet, Drawing>();
	public org.apache.poi.ss.usermodel.Cell POIAddCell (org.apache.poi.ss.usermodel.Sheet sheet, int iRow, int iCol, int iType, Object value, CellStyle style, String comment, String sAuthorCharset)
	{
		org.apache.poi.ss.usermodel.Row row = sheet.getRow (iRow);
		if (row == null) row = sheet.createRow (iRow);

		org.apache.poi.ss.usermodel.Cell cell = row.getCell (iCol, org.apache.poi.ss.usermodel.Row.CREATE_NULL_AS_BLANK);
		cell.setCellType (iType);
		switch (iType)
		{
			case org.apache.poi.ss.usermodel.Cell.CELL_TYPE_NUMERIC:	// 0
				cell.setCellValue (((Double)value).doubleValue());
				break;
			case org.apache.poi.ss.usermodel.Cell.CELL_TYPE_FORMULA:	// 2
				//cell.setCellType (org.apache.poi.ss.usermodel.Cell.CELL_TYPE_NUMERIC);
				cell.setCellFormula ((String)value);
				break;
			case org.apache.poi.ss.usermodel.Cell.CELL_TYPE_STRING:	// 1
			default:
				cell.setCellValue ((String)value);
				break;
		}

		if (comment != null && comment.length()>0)
		{
			org.apache.poi.ss.usermodel.Workbook workbook = sheet.getWorkbook ();
			CreationHelper factory = workbook.getCreationHelper ();
			ClientAnchor anchor = factory.createClientAnchor ();
				anchor.setDx1 (8);
				anchor.setDy1 (-4);
				anchor.setDx2 (8);
				anchor.setDy2 (+4);
				anchor.setCol1 (iCol+1);
				anchor.setRow1 (iRow==0 ? 0 : iRow-1);
				anchor.setCol2 (iCol+4);
				anchor.setRow2 (iRow==0 ? iRow+3 : iRow+2);
			Drawing drawing = drawingPatriarches.get (sheet);
			if (drawing == null)
			{
				drawing = sheet.createDrawingPatriarch ();
				drawingPatriarches.put (sheet, drawing);
			}
			Comment oComment = drawing.createCellComment (anchor);
			RichTextString richText = factory.createRichTextString (comment);
			oComment.setString (richText);
			String sAuthor = "Chinese中文";
			try
			{
				if (sAuthorCharset!=null && sAuthorCharset.length() != 0) sAuthor = new String(sAuthor.getBytes(), sAuthorCharset);

				// dump
				System.out.println (sAuthor);
				byte[] buf = (sAuthorCharset!=null && sAuthorCharset.length() != 0) ? sAuthor.getBytes(sAuthorCharset) : sAuthor.getBytes();
				for (int i=0; i<buf.length; i++)
				{
					System.out.print (String.format("%02X", buf[i]&0xFF));
					System.out.print (" ");
				}
				System.out.println ();
			}
			catch (java.io.UnsupportedEncodingException e)
			{
				e.printStackTrace (System.err);
			}

			oComment.setAuthor (sAuthor);
			cell.setCellComment (oComment);
		}

		if (style != null)
		{
			cell.setCellStyle (style);
		}
		return cell;
	}

	public org.apache.poi.ss.usermodel.Cell POIAddNumberCell (org.apache.poi.ss.usermodel.Sheet sheet, int iRow, int iCol, double value, CellStyle style, String comment, String sAuthorCharset)
	{
		return POIAddCell (sheet, iRow, iCol, org.apache.poi.ss.usermodel.Cell.CELL_TYPE_NUMERIC, value, style, comment, sAuthorCharset);
	}

	public org.apache.poi.ss.usermodel.Cell POIAddStringCell (org.apache.poi.ss.usermodel.Sheet sheet, int iRow, int iCol, String value, CellStyle style, String comment, String sAuthorCharset)
	{
		return POIAddCell (sheet, iRow, iCol, org.apache.poi.ss.usermodel.Cell.CELL_TYPE_STRING, value, style, comment, sAuthorCharset);
	}

	public org.apache.poi.ss.usermodel.Cell POIAddFormulaCell (org.apache.poi.ss.usermodel.Sheet sheet, int iRow, int iCol, String value, CellStyle style, String comment, String sAuthorCharset)
	{
		return POIAddCell (sheet, iRow, iCol, org.apache.poi.ss.usermodel.Cell.CELL_TYPE_FORMULA, value, style, comment, sAuthorCharset);
	}
}